@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -106,16 +106,12 @@ static void at_csca_set(struct ofono_sms *sms,
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 	char buf[64];
 
-	if (cbd == NULL)
-		goto error;
-
 	snprintf(buf, sizeof(buf), "AT+CSCA=\"%s\",%d", sca->number, sca->type);
 
 	if (g_at_chat_send(data->chat, buf, csca_prefix,
 				at_csca_set_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, user_data);
@@ -173,14 +169,10 @@ static void at_csca_query(struct ofono_sms *sms, ofono_sms_sca_query_cb_t cb,
 	struct sms_data *data = ofono_sms_get_data(sms);
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 
-	if (cbd == NULL)
-		goto error;
-
 	if (g_at_chat_send(data->chat, "AT+CSCA?", csca_prefix,
 				at_csca_query_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, NULL, user_data);
@@ -227,9 +219,6 @@ static void at_cmgs(struct ofono_sms *sms, unsigned char *pdu, int pdu_len,
 	char buf[512];
 	int len;
 
-	if (cbd == NULL)
-		goto error;
-
 	if (mms) {
 		snprintf(buf, sizeof(buf), "AT+CMMS=%d", mms);
 		g_at_chat_send(data->chat, buf, none_prefix,
@@ -243,7 +232,6 @@ static void at_cmgs(struct ofono_sms *sms, unsigned char *pdu, int pdu_len,
 				at_cmgs_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, user_data);
@@ -267,16 +255,12 @@ static void at_cgsms_set(struct ofono_sms *sms, int bearer,
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 	char buf[64];
 
-	if (cbd == NULL)
-		goto error;
-
 	snprintf(buf, sizeof(buf), "AT+CGSMS=%d", bearer);
 
 	if (g_at_chat_send(data->chat, buf, none_prefix,
 				at_cgsms_set_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, user_data);
@@ -319,14 +303,10 @@ static void at_cgsms_query(struct ofono_sms *sms,
 	struct sms_data *data = ofono_sms_get_data(sms);
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 
-	if (cbd == NULL)
-		goto error;
-
 	if (g_at_chat_send(data->chat, "AT+CGSMS?", cgsms_prefix,
 				at_cgsms_query_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, user_data);
@@ -798,15 +778,21 @@ static gboolean build_cnmi_string(char *buf, int *cnmi_opts,
 
 	DBG("");
 
-	if (data->vendor == OFONO_VENDOR_QUALCOMM_MSM ||
-			data->vendor == OFONO_VENDOR_HUAWEI ||
-			data->vendor == OFONO_VENDOR_NOVATEL)
+	switch (data->vendor) {
+	case OFONO_VENDOR_GOBI:
+	case OFONO_VENDOR_QUALCOMM_MSM:
+	case OFONO_VENDOR_NOVATEL:
+	case OFONO_VENDOR_HUAWEI:
+	case OFONO_VENDOR_ZTE:
 		/* MSM devices advertise support for mode 2, but return an
 		 * error if we attempt to actually use it. */
 		mode = "1";
-	else
+		break;
+	default:
 		/* Sounds like 2 is the sanest mode */
 		mode = "2310";
+		break;
+	}
 
 	if (!append_cnmi_element(buf, &len, cnmi_opts[0], mode, FALSE))
 		return FALSE;
@@ -826,10 +812,14 @@ static gboolean build_cnmi_string(char *buf, int *cnmi_opts,
 	 * ack it with error "CNMA not expected."  However, not acking it
 	 * sends the device into la-la land.
 	 */
-	if (data->vendor == OFONO_VENDOR_NOVATEL)
+	switch (data->vendor) {
+	case OFONO_VENDOR_NOVATEL:
 		mode = "20";
-	else
+		break;
+	default:
 		mode = "120";
+		break;
+	}
 
 	/*
 	 * Try to deliver Status-Reports via +CDS, then CDSI or don't
@@ -916,11 +906,17 @@ static void at_cnmi_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		supported = TRUE;
 
 	/* support for ack pdu is not working */
-	if (data->vendor == OFONO_VENDOR_IFX ||
-			data->vendor == OFONO_VENDOR_HUAWEI ||
-			data->vendor == OFONO_VENDOR_NOVATEL ||
-			data->vendor == OFONO_VENDOR_OPTION_HSO)
+	switch (data->vendor) {
+	case OFONO_VENDOR_IFX:
+	case OFONO_VENDOR_GOBI:
+	case OFONO_VENDOR_ZTE:
+	case OFONO_VENDOR_HUAWEI:
+	case OFONO_VENDOR_NOVATEL:
+	case OFONO_VENDOR_OPTION_HSO:
 		goto out;
+	default:
+		break;
+	}
 
 	if (data->cnma_enabled)
 		construct_ack_pdu(data);
@@ -1138,13 +1134,16 @@ static void at_csms_status_cb(gboolean ok, GAtResult *result,
 			goto out;
 
 
-		if (data->vendor == OFONO_VENDOR_HUAWEI ||
-				data->vendor == OFONO_VENDOR_NOVATEL) {
+		switch (data->vendor) {
+		case OFONO_VENDOR_HUAWEI:
+		case OFONO_VENDOR_NOVATEL:
 			g_at_result_iter_skip_next(&iter);
 			service = 0;
-		} else {
+			break;
+		default:
 			if (!g_at_result_iter_next_number(&iter, &service))
 				goto out;
+			break;
 		}
 
 		if (!g_at_result_iter_next_number(&iter, &mt))
