@@ -59,6 +59,9 @@ struct indicator {
 	int value;
 	int min;
 	int max;
+	gboolean deferred;
+	gboolean active;
+	gboolean mandatory;
 };
 
 static void emulator_debug(const char *str, void *data)
@@ -345,8 +348,33 @@ static struct ofono_call *find_call_with_status(struct ofono_emulator *em,
 	return __ofono_voicecall_find_call_with_status(vc, status);
 }
 
-static void notify_ccwa(struct ofono_emulator *em)
+static void notify_deferred_indicators(GAtServer *server, void *user_data)
 {
+	struct ofono_emulator *em = user_data;
+	int i;
+	char buf[20];
+	GSList *l;
+	struct indicator *ind;
+
+	for (i = 1, l = em->indicators; l; l = l->next, i++) {
+		ind = l->data;
+
+		if (!ind->deferred)
+			continue;
+
+		if (em->events_mode == 3 && em->events_ind && em->slc &&
+				ind->active) {
+			sprintf(buf, "+CIEV: %d,%d", i, ind->value);
+			g_at_server_send_unsolicited(em->server, buf);
+		}
+
+		ind->deferred = FALSE;
+	}
+}
+
+static gboolean notify_ccwa(void *user_data)
+{
+	struct ofono_emulator *em = user_data;
 	struct ofono_call *c;
 	const char *phone;
 	/*
@@ -355,8 +383,9 @@ static void notify_ccwa(struct ofono_emulator *em)
 	 */
 	char str[OFONO_MAX_PHONE_NUMBER_LENGTH + 14 + 1];
 
-	if (!em->ccwa)
-		return;
+	if ((em->type == OFONO_EMULATOR_TYPE_HFP && em->slc == FALSE) ||
+			!em->ccwa)
+		goto end;
 
 	c = find_call_with_status(em, CALL_STATUS_WAITING);
 
@@ -367,10 +396,16 @@ static void notify_ccwa(struct ofono_emulator *em)
 		g_at_server_send_unsolicited(em->server, str);
 	} else
 		g_at_server_send_unsolicited(em->server, "+CCWA: \"\",128");
+
+end:
+	em->callsetup_source = 0;
+
+	return FALSE;
 }
 
-static void notify_ring(struct ofono_emulator *em)
+static gboolean notify_ring(void *user_data)
 {
+	struct ofono_emulator *em = user_data;
 	struct ofono_call *c;
 	const char *phone;
 	/*
@@ -379,15 +414,25 @@ static void notify_ring(struct ofono_emulator *em)
 	 */
 	char str[OFONO_MAX_PHONE_NUMBER_LENGTH + 14 + 1];
 
+	if (em->type == OFONO_EMULATOR_TYPE_HFP && em->slc == FALSE)
+		return TRUE;
+
 	g_at_server_send_unsolicited(em->server, "RING");
 
 	if (!em->clip)
-		return;
+		return TRUE;
 
 	c = find_call_with_status(em, CALL_STATUS_INCOMING);
 
+	/*
+	 * In case of waiting call becoming an incoming call, call status
+	 * change may not have been done yet, so try to find waiting call too
+	 */
 	if (c == NULL)
-		return;
+		c = find_call_with_status(em, CALL_STATUS_WAITING);
+
+	if (c == NULL)
+		return TRUE;
 
 	switch (c->clip_validity) {
 	case CLIP_VALIDITY_VALID:
@@ -400,22 +445,6 @@ static void notify_ring(struct ofono_emulator *em)
 		g_at_server_send_unsolicited(em->server, "+CLIP: \"\",128");
 		break;
 	}
-}
-
-static gboolean send_callsetup_notification(gpointer user_data)
-{
-	struct ofono_emulator *em = user_data;
-	struct indicator *call_ind;
-
-	if (em->type == OFONO_EMULATOR_TYPE_HFP && em->slc == FALSE)
-		return TRUE;
-
-	call_ind = find_indicator(em, OFONO_EMULATOR_IND_CALL, NULL);
-
-	if (call_ind->value == OFONO_EMULATOR_CALL_INACTIVE)
-		notify_ring(em);
-	else
-		notify_ccwa(em);
 
 	return TRUE;
 }
@@ -672,6 +701,8 @@ static void ccwa_cb(GAtServer *server, GAtServerRequestType type,
 	struct ofono_emulator *em = user_data;
 	GAtResultIter iter;
 	int val;
+	struct indicator *call_ind;
+	struct indicator *cs_ind;
 
 	if (em->slc == FALSE)
 		goto fail;
@@ -691,6 +722,15 @@ static void ccwa_cb(GAtServer *server, GAtServerRequestType type,
 		if (g_at_result_iter_skip_next(&iter))
 			goto fail;
 
+		call_ind = find_indicator(em, OFONO_EMULATOR_IND_CALL, NULL);
+		cs_ind = find_indicator(em, OFONO_EMULATOR_IND_CALLSETUP, NULL);
+
+		if (cs_ind->value == OFONO_EMULATOR_CALLSETUP_INCOMING &&
+				call_ind->value == OFONO_EMULATOR_CALL_ACTIVE &&
+				em->ccwa == FALSE && val == 1)
+			em->callsetup_source = g_timeout_add_seconds(0,
+							notify_ccwa, em);
+
 		em->ccwa = val;
 
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
@@ -746,8 +786,61 @@ fail:
 	}
 }
 
+static void bia_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *result, gpointer user_data)
+{
+	struct ofono_emulator *em = user_data;
+
+	switch (type) {
+	case G_AT_SERVER_REQUEST_TYPE_SET:
+	{
+		GAtResultIter iter;
+		GSList *l;
+		int val;
+
+		g_at_result_iter_init(&iter, result);
+		g_at_result_iter_next(&iter, "");
+
+		/* check validity of the request */
+		while (g_at_result_iter_next_number_default(&iter, 0, &val))
+			if (val != 0 &&  val != 1)
+				goto fail;
+
+		/* Check that we have no non-numbers in the stream */
+		if (g_at_result_iter_skip_next(&iter) == TRUE)
+			goto fail;
+
+		/* request is valid, update the indicator activation status */
+		g_at_result_iter_init(&iter, result);
+		g_at_result_iter_next(&iter, "");
+
+		for (l = em->indicators; l; l = l->next) {
+			struct indicator *ind = l->data;
+
+			if (g_at_result_iter_next_number_default(&iter,
+						ind->active, &val) == FALSE)
+				break;
+
+			if (ind->mandatory == TRUE)
+				continue;
+
+			ind->active = val;
+		}
+
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
+		break;
+	}
+
+	default:
+fail:
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
+		break;
+	}
+}
+
 static void emulator_add_indicator(struct ofono_emulator *em, const char* name,
-					int min, int max, int dflt)
+					int min, int max, int dflt,
+					gboolean mandatory)
 {
 	struct indicator *ind;
 
@@ -761,6 +854,8 @@ static void emulator_add_indicator(struct ofono_emulator *em, const char* name,
 	ind->min = min;
 	ind->max = max;
 	ind->value = dflt;
+	ind->active = TRUE;
+	ind->mandatory = mandatory;
 
 	em->indicators = g_slist_append(em->indicators, ind);
 }
@@ -819,17 +914,24 @@ void ofono_emulator_register(struct ofono_emulator *em, int fd)
 	g_at_server_set_debug(em->server, emulator_debug, "Server");
 	g_at_server_set_disconnect_function(em->server,
 						emulator_disconnect, em);
+	g_at_server_set_finish_callback(em->server, notify_deferred_indicators,
+						em);
 
 	if (em->type == OFONO_EMULATOR_TYPE_HFP) {
-		emulator_add_indicator(em, OFONO_EMULATOR_IND_SERVICE, 0, 1, 0);
-		emulator_add_indicator(em, OFONO_EMULATOR_IND_CALL, 0, 1, 0);
+		emulator_add_indicator(em, OFONO_EMULATOR_IND_SERVICE, 0, 1, 0,
+									FALSE);
+		emulator_add_indicator(em, OFONO_EMULATOR_IND_CALL, 0, 1, 0,
+									TRUE);
 		emulator_add_indicator(em, OFONO_EMULATOR_IND_CALLSETUP, 0, 3,
-									0);
+								0, TRUE);
 		emulator_add_indicator(em, OFONO_EMULATOR_IND_CALLHELD, 0, 2,
-									0);
-		emulator_add_indicator(em, OFONO_EMULATOR_IND_SIGNAL, 0, 5, 0);
-		emulator_add_indicator(em, OFONO_EMULATOR_IND_ROAMING, 0, 1, 0);
-		emulator_add_indicator(em, OFONO_EMULATOR_IND_BATTERY, 0, 5, 5);
+								0, TRUE);
+		emulator_add_indicator(em, OFONO_EMULATOR_IND_SIGNAL, 0, 5, 0,
+									FALSE);
+		emulator_add_indicator(em, OFONO_EMULATOR_IND_ROAMING, 0, 1, 0,
+									FALSE);
+		emulator_add_indicator(em, OFONO_EMULATOR_IND_BATTERY, 0, 5, 5,
+									FALSE);
 
 		g_at_server_register(em->server, "+BRSF", brsf_cb, em, NULL);
 		g_at_server_register(em->server, "+CIND", cind_cb, em, NULL);
@@ -837,6 +939,7 @@ void ofono_emulator_register(struct ofono_emulator *em, int fd)
 		g_at_server_register(em->server, "+CLIP", clip_cb, em, NULL);
 		g_at_server_register(em->server, "+CCWA", ccwa_cb, em, NULL);
 		g_at_server_register(em->server, "+CMEE", cmee_cb, em, NULL);
+		g_at_server_register(em->server, "+BIA", bia_cb, em, NULL);
 	}
 
 	__ofono_atom_register(em->atom, emulator_unregister);
@@ -1081,7 +1184,10 @@ void ofono_emulator_set_indicator(struct ofono_emulator *em,
 	char buf[20];
 	struct indicator *ind;
 	struct indicator *call_ind;
+	struct indicator *cs_ind;
+	gboolean call;
 	gboolean callsetup;
+	gboolean waiting;
 
 	ind = find_indicator(em, name, &i);
 
@@ -1092,45 +1198,60 @@ void ofono_emulator_set_indicator(struct ofono_emulator *em,
 	ind->value = value;
 
 	call_ind = find_indicator(em, OFONO_EMULATOR_IND_CALL, NULL);
+	cs_ind = find_indicator(em, OFONO_EMULATOR_IND_CALLSETUP, NULL);
 
-	callsetup = g_str_equal(name, OFONO_EMULATOR_IND_CALLSETUP);
+	call = ind == call_ind;
+	callsetup = ind == cs_ind;
 
 	/*
 	 * When callsetup indicator goes to Incoming and there is an active
 	 * call a +CCWA should be sent before +CIEV
 	 */
-	if (callsetup && value == OFONO_EMULATOR_CALLSETUP_INCOMING &&
-			call_ind->value == OFONO_EMULATOR_CALL_ACTIVE)
-		send_callsetup_notification(em);
+	waiting = (callsetup && value == OFONO_EMULATOR_CALLSETUP_INCOMING &&
+			call_ind->value == OFONO_EMULATOR_CALL_ACTIVE);
 
-	if (em->events_mode == 3 && em->events_ind && em->slc) {
-		sprintf(buf, "+CIEV: %d,%d", i, ind->value);
-		g_at_server_send_unsolicited(em->server, buf);
+	if (waiting)
+		notify_ccwa(em);
+
+	if (em->events_mode == 3 && em->events_ind && em->slc && ind->active) {
+		if (!g_at_server_command_pending(em->server)) {
+			sprintf(buf, "+CIEV: %d,%d", i, ind->value);
+			g_at_server_send_unsolicited(em->server, buf);
+		} else
+			ind->deferred = TRUE;
 	}
 
 	/*
-	 * Ring timer should be started when callsetup indicator is set to
-	 * Incoming.  If there is no active call, a first RING should be
-	 * sent just after the +CIEV and should be stopped for all other
-	 * values of callsetup
+	 * Ring timer should be started when:
+	 * - callsetup indicator is set to Incoming and there is no active call
+	 *   (not a waiting call)
+	 * - or call indicator is set to inactive while callsetup is already
+	 *   set to Incoming.
+	 * In those cases, a first RING should be sent just after the +CIEV
+	 * Ring timer should be stopped for all other values of callsetup
 	 */
+	if (waiting)
+		return;
+
+	/* Call state went from active/held + waiting -> incoming */
+	if (call && value == OFONO_EMULATOR_CALL_INACTIVE &&
+			cs_ind->value == OFONO_EMULATOR_CALLSETUP_INCOMING)
+		goto start_ring;
+
 	if (!callsetup)
 		return;
 
-	switch (value) {
-	case OFONO_EMULATOR_CALLSETUP_INCOMING:
-		if (call_ind->value == OFONO_EMULATOR_CALL_INACTIVE)
-			send_callsetup_notification(em);
-
-		em->callsetup_source = g_timeout_add_seconds(RING_TIMEOUT,
-					send_callsetup_notification, em);
-		break;
-	default:
+	if (value != OFONO_EMULATOR_CALLSETUP_INCOMING) {
 		if (em->callsetup_source > 0) {
 			g_source_remove(em->callsetup_source);
 			em->callsetup_source = 0;
 		}
 
-		break;
+		return;
 	}
+
+start_ring:
+	notify_ring(em);
+	em->callsetup_source = g_timeout_add_seconds(RING_TIMEOUT,
+							notify_ring, em);
 }
