@@ -37,8 +37,13 @@
 #include "util.h"
 #include "storage.h"
 
-#define NETWORK_REGISTRATION_FLAG_HOME_SHOW_PLMN 0x1
-#define NETWORK_REGISTRATION_FLAG_ROAMING_SHOW_SPN 0x2
+#define SETTINGS_STORE "netreg"
+#define SETTINGS_GROUP "Settings"
+
+#define NETWORK_REGISTRATION_FLAG_HOME_SHOW_PLMN	0x1
+#define NETWORK_REGISTRATION_FLAG_ROAMING_SHOW_SPN	0x2
+#define NETWORK_REGISTRATION_FLAG_READING_PNN		0x4
+#define NETWORK_REGISTRATION_FLAG_READING_SPN		0x8
 
 enum network_registration_mode {
 	NETWORK_REGISTRATION_MODE_AUTO =	0,
@@ -46,11 +51,6 @@ enum network_registration_mode {
 	NETWORK_REGISTRATION_MODE_AUTO_ONLY =	5, /* Out of range of 27.007 */
 };
 
-#define SETTINGS_STORE "netreg"
-#define SETTINGS_GROUP "Settings"
-
-static GSList *g_drivers = NULL;
-
 /* 27.007 Section 7.3 <stat> */
 enum operator_status {
 	OPERATOR_STATUS_UNKNOWN =	0,
@@ -72,7 +72,6 @@ struct ofono_netreg {
 	int flags;
 	DBusMessage *pending;
 	int signal_strength;
-	char *spname;
 	struct sim_spdi *spdi;
 	struct sim_eons *eons;
 	struct ofono_sim *sim;
@@ -84,6 +83,7 @@ struct ofono_netreg {
 	void *driver_data;
 	struct ofono_atom *atom;
 	unsigned int hfp_watch;
+	char *spn;
 };
 
 struct network_operator_data {
@@ -96,6 +96,8 @@ struct network_operator_data {
 	struct ofono_netreg *netreg;
 };
 
+static GSList *g_drivers = NULL;
+
 static const char *registration_mode_to_string(int mode)
 {
 	switch (mode) {
@@ -390,7 +392,7 @@ static char *get_operator_display_name(struct ofono_netreg *netreg)
 	if (opd->eons_info && opd->eons_info->longname)
 		plmn = opd->eons_info->longname;
 
-	if (netreg->spname == NULL || strlen(netreg->spname) == 0) {
+	if (netreg->spn == NULL || strlen(netreg->spn) == 0) {
 		g_strlcpy(name, plmn, len);
 		return name;
 	}
@@ -404,14 +406,14 @@ static char *get_operator_display_name(struct ofono_netreg *netreg)
 	if (home_or_spdi)
 		if (netreg->flags & NETWORK_REGISTRATION_FLAG_HOME_SHOW_PLMN)
 			/* Case 1 */
-			snprintf(name, len, "%s (%s)", netreg->spname, plmn);
+			snprintf(name, len, "%s (%s)", netreg->spn, plmn);
 		else
 			/* Case 2 */
-			snprintf(name, len, "%s", netreg->spname);
+			snprintf(name, len, "%s", netreg->spn);
 	else
 		if (netreg->flags & NETWORK_REGISTRATION_FLAG_ROAMING_SHOW_SPN)
 			/* Case 3 */
-			snprintf(name, len, "%s (%s)", netreg->spname, plmn);
+			snprintf(name, len, "%s (%s)", netreg->spn, plmn);
 		else
 			/* Case 4 */
 			snprintf(name, len, "%s", plmn);
@@ -419,13 +421,22 @@ static char *get_operator_display_name(struct ofono_netreg *netreg)
 	return name;
 }
 
+static void netreg_emit_operator_display_name(struct ofono_netreg *netreg)
+{
+	const char *operator = get_operator_display_name(netreg);
+
+	ofono_dbus_signal_property_changed(ofono_dbus_get_connection(),
+					__ofono_atom_get_path(netreg->atom),
+					OFONO_NETWORK_REGISTRATION_INTERFACE,
+					"Name", DBUS_TYPE_STRING, &operator);
+}
+
 static void set_network_operator_name(struct network_operator_data *opd,
 					const char *name)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	struct ofono_netreg *netreg = opd->netreg;
 	const char *path;
-	const char *operator;
 
 	if (name[0] == '\0')
 		return;
@@ -443,16 +454,8 @@ static void set_network_operator_name(struct network_operator_data *opd,
 	if (opd->eons_info && opd->eons_info->longname)
 		return;
 
-	if (opd == netreg->current_operator) {
-		const char *path = __ofono_atom_get_path(netreg->atom);
-
-		operator = get_operator_display_name(netreg);
-
-		ofono_dbus_signal_property_changed(conn, path,
-					OFONO_NETWORK_REGISTRATION_INTERFACE,
-					"Name", DBUS_TYPE_STRING,
-					&operator);
-	}
+	if (opd == netreg->current_operator)
+		netreg_emit_operator_display_name(netreg);
 
 	/* Don't emit when only operator name is reported */
 	if (opd->mcc[0] == '\0' && opd->mnc[0] == '\0')
@@ -498,16 +501,8 @@ static void set_network_operator_eons_info(struct network_operator_data *opd,
 					OFONO_NETWORK_OPERATOR_INTERFACE,
 					"Name", DBUS_TYPE_STRING, &newname);
 
-		if (opd == netreg->current_operator) {
-			const char *npath = __ofono_atom_get_path(netreg->atom);
-			const char *operator =
-				get_operator_display_name(netreg);
-
-			ofono_dbus_signal_property_changed(conn, npath,
-					OFONO_NETWORK_REGISTRATION_INTERFACE,
-					"Name", DBUS_TYPE_STRING,
-					&operator);
-		}
+		if (opd == netreg->current_operator)
+			netreg_emit_operator_display_name(netreg);
 	}
 
 	if (old_eons_info && old_eons_info->info)
@@ -1209,7 +1204,6 @@ static void current_operator_callback(const struct ofono_error *error,
 	struct ofono_netreg *netreg = data;
 	const char *path = __ofono_atom_get_path(netreg->atom);
 	GSList *op = NULL;
-	const char *operator;
 
 	DBG("%p, %p", netreg, netreg->current_operator);
 
@@ -1279,12 +1273,7 @@ static void current_operator_callback(const struct ofono_error *error,
 	}
 
 emit:
-	operator = get_operator_display_name(netreg);
-
-	ofono_dbus_signal_property_changed(conn, path,
-					OFONO_NETWORK_REGISTRATION_INTERFACE,
-					"Name", DBUS_TYPE_STRING,
-					&operator);
+	netreg_emit_operator_display_name(netreg);
 
 	if (netreg->current_operator) {
 		if (netreg->current_operator->mcc[0] != '\0') {
@@ -1598,7 +1587,7 @@ static void sim_pnn_read_cb(int ok, int length, int record,
 		goto check;
 
 	if (length < 3 || record_length < 3 || length < record_length)
-		return;
+		goto check;
 
 	total = length / record_length;
 
@@ -1611,6 +1600,8 @@ static void sim_pnn_read_cb(int ok, int length, int record,
 		return;
 
 check:
+	netreg->flags &= ~NETWORK_REGISTRATION_FLAG_READING_PNN;
+
 	/*
 	 * If PNN is not present then OPL is not useful, don't
 	 * retrieve it.  If OPL is not there then PNN[1] will
@@ -1628,47 +1619,39 @@ static void sim_spdi_read_cb(int ok, int length, int record,
 				int record_length, void *user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct network_operator_data *current = netreg->current_operator;
 
 	if (!ok)
 		return;
 
 	netreg->spdi = sim_spdi_new(data, length);
 
-	if (current == NULL)
+	if (netreg->current_operator == NULL)
 		return;
 
-	if (netreg->status == NETWORK_REGISTRATION_STATUS_ROAMING) {
-		DBusConnection *conn = ofono_dbus_get_connection();
-		const char *path = __ofono_atom_get_path(netreg->atom);
-		const char *operator;
+	if (netreg->status != NETWORK_REGISTRATION_STATUS_ROAMING)
+		return;
 
-		if (!sim_spdi_lookup(netreg->spdi,
-					current->mcc, current->mnc))
-			return;
+	if (!sim_spdi_lookup(netreg->spdi, netreg->current_operator->mcc,
+				netreg->current_operator->mnc))
+		return;
 
-		operator = get_operator_display_name(netreg);
-
-		ofono_dbus_signal_property_changed(conn, path,
-					OFONO_NETWORK_REGISTRATION_INTERFACE,
-					"Name", DBUS_TYPE_STRING,
-					&operator);
-	}
+	netreg_emit_operator_display_name(netreg);
 }
 
-static void sim_spn_read_cb(int ok, int length, int record,
-				const unsigned char *data,
-				int record_length, void *user_data)
+static void sim_spn_display_condition_parse(struct ofono_netreg *netreg,
+						guint8 dcbyte)
+{
+	if (dcbyte & SIM_EFSPN_DC_HOME_PLMN_BIT)
+		netreg->flags |= NETWORK_REGISTRATION_FLAG_HOME_SHOW_PLMN;
+
+	if (!(dcbyte & SIM_EFSPN_DC_ROAMING_SPN_BIT))
+		netreg->flags |= NETWORK_REGISTRATION_FLAG_ROAMING_SHOW_SPN;
+}
+
+static gboolean sim_spn_parse(const void *data, int length, char **dst)
 {
-	struct ofono_netreg *netreg = user_data;
-	unsigned char dcbyte;
 	char *spn;
 
-	if (!ok)
-		return;
-
-	dcbyte = data[0];
-
 	/*
 	 * TS 31.102 says:
 	 *
@@ -1685,40 +1668,120 @@ static void sim_spn_read_cb(int ok, int length, int record,
 	 * itself which is not there either.  11.11 contains the same
 	 * paragraph as 51.101 and has an Annex B which we implement.
 	 */
-	spn = sim_string_to_utf8(data + 1, length - 1);
+	spn = sim_string_to_utf8(data, length);
 	if (spn == NULL) {
 		ofono_error("EFspn read successfully, but couldn't parse");
-		return;
+		return FALSE;
 	}
 
 	if (strlen(spn) == 0) {
 		g_free(spn);
+		return FALSE;
+	}
+
+	*dst = spn;
+	return TRUE;
+}
+
+static void sim_cphs_spn_short_read_cb(int ok, int length, int record,
+					const unsigned char *data,
+					int record_length, void *user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+
+	netreg->flags &= ~NETWORK_REGISTRATION_FLAG_READING_SPN;
+
+	if (!ok)
+		return;
+
+	if (!sim_spn_parse(data, length, &netreg->spn))
+		return;
+
+	if (netreg->current_operator)
+		netreg_emit_operator_display_name(netreg);
+}
+
+static void sim_cphs_spn_read_cb(int ok, int length, int record,
+					const unsigned char *data,
+					int record_length, void *user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+
+	if (!ok) {
+		if (__ofono_sim_cphs_service_available(netreg->sim,
+						SIM_CPHS_SERVICE_SHORT_SPN))
+			ofono_sim_read(netreg->sim_context,
+					SIM_EF_CPHS_SPN_SHORT_FILEID,
+					OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+					sim_cphs_spn_short_read_cb, netreg);
+		else
+			netreg->flags &= ~NETWORK_REGISTRATION_FLAG_READING_SPN;
+
 		return;
 	}
 
-	netreg->spname = spn;
-	ofono_sim_read(netreg->sim_context, SIM_EFSPDI_FILEID,
-			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
-			sim_spdi_read_cb, netreg);
+	netreg->flags &= ~NETWORK_REGISTRATION_FLAG_READING_SPN;
 
-	if (dcbyte & SIM_EFSPN_DC_HOME_PLMN_BIT)
-		netreg->flags |= NETWORK_REGISTRATION_FLAG_HOME_SHOW_PLMN;
+	if (!sim_spn_parse(data, length, &netreg->spn))
+		return;
 
-	if (!(dcbyte & SIM_EFSPN_DC_ROAMING_SPN_BIT))
-		netreg->flags |= NETWORK_REGISTRATION_FLAG_ROAMING_SHOW_SPN;
+	if (netreg->current_operator)
+		netreg_emit_operator_display_name(netreg);
+}
 
-	if (netreg->current_operator) {
-		DBusConnection *conn = ofono_dbus_get_connection();
-		const char *path = __ofono_atom_get_path(netreg->atom);
-		const char *operator;
+static void sim_spn_read_cb(int ok, int length, int record,
+				const unsigned char *data,
+				int record_length, void *user_data)
+{
+	struct ofono_netreg *netreg = user_data;
 
-		operator = get_operator_display_name(netreg);
+	if (!ok) {
+		ofono_sim_read(netreg->sim_context,
+				SIM_EF_CPHS_SPN_FILEID,
+				OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+				sim_cphs_spn_read_cb, netreg);
 
-		ofono_dbus_signal_property_changed(conn, path,
-					OFONO_NETWORK_REGISTRATION_INTERFACE,
-					"Name", DBUS_TYPE_STRING,
-					&operator);
+		return;
 	}
+
+	netreg->flags &= ~NETWORK_REGISTRATION_FLAG_READING_SPN;
+
+	if (!sim_spn_parse(data + 1, length - 1, &netreg->spn))
+		return;
+
+	sim_spn_display_condition_parse(netreg, data[0]);
+
+	if (netreg->current_operator)
+		netreg_emit_operator_display_name(netreg);
+}
+
+static void sim_spn_changed(int id, void *userdata)
+{
+	struct ofono_netreg *netreg = userdata;
+	gboolean had_spn;
+
+	if (netreg->flags & NETWORK_REGISTRATION_FLAG_READING_SPN)
+		return;
+
+	had_spn = netreg->spn != NULL && strlen(netreg->spn) > 0;
+	netreg->flags &= ~(NETWORK_REGISTRATION_FLAG_HOME_SHOW_PLMN |
+				NETWORK_REGISTRATION_FLAG_ROAMING_SHOW_SPN);
+
+	g_free(netreg->spn);
+	netreg->spn = NULL;
+
+	/*
+	 * We can't determine whether the property really changed
+	 * without checking the name, before and after.  Instead we use a
+	 * simple heuristic, which will not always be correct
+	 */
+	if (had_spn && netreg->current_operator)
+		netreg_emit_operator_display_name(netreg);
+
+	netreg->flags |= NETWORK_REGISTRATION_FLAG_READING_SPN;
+	ofono_sim_read(netreg->sim_context, SIM_EFSPN_FILEID,
+			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+			sim_spn_read_cb, netreg);
 }
 
 int ofono_netreg_get_location(struct ofono_netreg *netreg)
@@ -1831,8 +1894,10 @@ static void netreg_unregister(struct ofono_atom *atom)
 	for (l = netreg->operator_list; l; l = l->next) {
 		struct network_operator_data *opd = l->data;
 
-		if (opd->mcc[0] == '\0' && opd->mnc[0] == '\0')
+		if (opd->mcc[0] == '\0' && opd->mnc[0] == '\0') {
+			g_free(opd);
 			continue;
+		}
 
 		network_operator_dbus_unregister(netreg, l->data);
 	}
@@ -1882,7 +1947,7 @@ static void netreg_remove(struct ofono_atom *atom)
 	sim_eons_free(netreg->eons);
 	sim_spdi_free(netreg->spdi);
 
-	g_free(netreg->spname);
+	g_free(netreg->spn);
 	g_free(netreg);
 }
 
@@ -1991,6 +2056,8 @@ static void sim_pnn_opl_changed(int id, void *userdata)
 	struct ofono_netreg *netreg = userdata;
 	GSList *l;
 
+	if (netreg->flags & NETWORK_REGISTRATION_FLAG_READING_PNN)
+		return;
 	/*
 	 * Free references to structures on the netreg->eons list and
 	 * update the operator info on D-bus.  If EFpnn/EFopl read succeeds,
@@ -2003,40 +2070,26 @@ static void sim_pnn_opl_changed(int id, void *userdata)
 	sim_eons_free(netreg->eons);
 	netreg->eons = NULL;
 
+	netreg->flags |= NETWORK_REGISTRATION_FLAG_READING_PNN;
 	ofono_sim_read(netreg->sim_context, SIM_EFPNN_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_FIXED,
 			sim_pnn_read_cb, netreg);
 }
 
-static void sim_spn_spdi_changed(int id, void *userdata)
+static void sim_spdi_changed(int id, void *userdata)
 {
 	struct ofono_netreg *netreg = userdata;
 
-	netreg->flags &= ~(NETWORK_REGISTRATION_FLAG_HOME_SHOW_PLMN |
-			NETWORK_REGISTRATION_FLAG_ROAMING_SHOW_SPN);
-
-	g_free(netreg->spname);
-	netreg->spname = NULL;
-
 	sim_spdi_free(netreg->spdi);
 	netreg->spdi = NULL;
 
-	if (netreg->current_operator) {
-		DBusConnection *conn = ofono_dbus_get_connection();
-		const char *path = __ofono_atom_get_path(netreg->atom);
-		const char *operator;
+	if (netreg->current_operator &&
+			netreg->status == NETWORK_REGISTRATION_STATUS_ROAMING)
+		netreg_emit_operator_display_name(netreg);
 
-		operator = get_operator_display_name(netreg);
-
-		ofono_dbus_signal_property_changed(conn, path,
-					OFONO_NETWORK_REGISTRATION_INTERFACE,
-					"Name", DBUS_TYPE_STRING,
-					&operator);
-	}
-
-	ofono_sim_read(netreg->sim_context, SIM_EFSPN_FILEID,
+	ofono_sim_read(netreg->sim_context, SIM_EFSPDI_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
-			sim_spn_read_cb, netreg);
+			sim_spdi_read_cb, netreg);
 }
 
 static void emulator_cops_cb(struct ofono_emulator *em,
@@ -2135,6 +2188,7 @@ void ofono_netreg_register(struct ofono_netreg *netreg)
 
 		netreg_load_settings(netreg);
 
+		netreg->flags |= NETWORK_REGISTRATION_FLAG_READING_PNN;
 		ofono_sim_read(netreg->sim_context, SIM_EFPNN_FILEID,
 				OFONO_SIM_FILE_STRUCTURE_FIXED,
 				sim_pnn_read_cb, netreg);
@@ -2145,15 +2199,38 @@ void ofono_netreg_register(struct ofono_netreg *netreg)
 						sim_pnn_opl_changed, netreg,
 						NULL);
 
+		netreg->flags |= NETWORK_REGISTRATION_FLAG_READING_SPN;
 		ofono_sim_read(netreg->sim_context, SIM_EFSPN_FILEID,
 				OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 				sim_spn_read_cb, netreg);
+
 		ofono_sim_add_file_watch(netreg->sim_context, SIM_EFSPN_FILEID,
-						sim_spn_spdi_changed, netreg,
+						sim_spn_changed, netreg,
 						NULL);
-		ofono_sim_add_file_watch(netreg->sim_context, SIM_EFSPDI_FILEID,
-						sim_spn_spdi_changed, netreg,
+		ofono_sim_add_file_watch(netreg->sim_context,
+						SIM_EF_CPHS_SPN_FILEID,
+						sim_spn_changed, netreg,
 						NULL);
+
+		if (__ofono_sim_cphs_service_available(netreg->sim,
+						SIM_CPHS_SERVICE_SHORT_SPN))
+			ofono_sim_add_file_watch(netreg->sim_context,
+						SIM_EF_CPHS_SPN_SHORT_FILEID,
+						sim_spn_changed,
+						netreg, NULL);
+
+		if (__ofono_sim_service_available(netreg->sim,
+				SIM_UST_SERVICE_PROVIDER_DISPLAY_INFO,
+				SIM_SST_SERVICE_PROVIDER_DISPLAY_INFO)) {
+			ofono_sim_read(netreg->sim_context, SIM_EFSPDI_FILEID,
+					OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+					sim_spdi_read_cb, netreg);
+
+			ofono_sim_add_file_watch(netreg->sim_context,
+							SIM_EFSPDI_FILEID,
+							sim_spdi_changed,
+							netreg, NULL);
+		}
 	}
 
 	__ofono_atom_register(netreg->atom, netreg_unregister);
